/*
 * Copyright 2014 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.tooling.internal.protocol;

/**
 * Cancellation token passed from consumer to provider to propagate cancellation events.
 * @since 2.1-rc-1
 */
public interface InternalCancellationToken {
    boolean isCancellationRequested();

    /**
     * Adds a callback that will be executed when cancel event is triggered.
     * It can be run synchronously if the token is already cancelled.
     *
     * @param cancellationHandler callback to be executed when cancel is requested.
     * @return current state of cancellation request before callback was added.
     */
    boolean addCallback(Runnable cancellationHandler);

    /**
     * Removes a callback from a set of handlers called when cancel is requested.
     *
     * @param cancellationHandler removed callback.
     */
    void removeCallback(Runnable cancellationHandler);
}
