/*
 * Copyright 2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal;

import org.gradle.api.Action;
import org.gradle.api.specs.Spec;
import org.gradle.internal.service.scopes.Scope;
import org.gradle.internal.service.scopes.ServiceScope;

import javax.annotation.Nullable;

@ServiceScope(Scope.CrossBuildSession.class)
public interface CollectionCallbackActionDecorator {
    <T> Action<T> decorate(@Nullable Action<T> action);

    <T> Spec<T> decorateSpec(Spec<T> spec);

    CollectionCallbackActionDecorator NOOP = new CollectionCallbackActionDecorator() {
        @Override
        public <T> Action<T> decorate(@Nullable Action<T> action) {
            return action;
        }

        @Override
        public <T> Spec<T> decorateSpec(Spec<T> spec) {
            return spec;
        }
    };
}
