/*
 * Copyright 2022 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal;

import org.gradle.TaskExecutionRequest;

import javax.annotation.Nullable;
import java.io.File;
import java.io.Serializable;
import java.util.Collections;
import java.util.List;

public class RunDefaultTasksExecutionRequest implements TaskExecutionRequest, Serializable {
    @Nullable
    private final String projectPath;
    @Nullable
    private final File rootDir;

    public RunDefaultTasksExecutionRequest() {
        this(null, null);
    }

    public RunDefaultTasksExecutionRequest(@Nullable String projectPath, @Nullable File rootDir) {
        this.projectPath = projectPath;
        this.rootDir = rootDir;
    }

    @Override
    public List<String> getArgs() {
        return Collections.emptyList();
    }

    @Nullable
    @Override
    public String getProjectPath() {
        return projectPath;
    }

    @Nullable
    @Override
    public File getRootDir() {
        return rootDir;
    }
}
