/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.extensions.core

import org.gradle.api.Project
import org.gradle.api.plugins.ExtraPropertiesExtension
import org.gradle.internal.extensions.stdlib.uncheckedCast


inline fun <reified T : Any> Project.setSingletonProperty(value: T) {
    extra[T::class.java.name] = value
}


inline fun <reified T> Project.peekSingletonProperty(): T? =
    if (extra.has(T::class.java.name)) extra[T::class.java.name]?.uncheckedCast()
    else null


inline fun <reified T> Project.popSingletonProperty(): T? =
    extra.remove(T::class.java.name)?.uncheckedCast()


val Project.extra: ExtraPropertiesExtension
    get() = extensions.extraProperties
