/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal;

import org.gradle.api.Action;
import org.gradle.api.InvalidUserCodeException;

/**
 * A delegating {@link Action} implementation that wraps exceptions in InvalidUserCodeException.
 * Used to wrap a closure or action parameter that contains user-provided executable code.
 */
public class UserCodeAction<T> implements Action<T> {
    private final String exceptionMessage;
    private final Action<? super T> delegate;

    public UserCodeAction(String exceptionMessage, Action<? super T> delegate) {
        this.exceptionMessage = exceptionMessage;
        this.delegate = delegate;
    }

    @Override
    public void execute(T target) {
        try {
            delegate.execute(target);
        } catch (Exception e) {
            throw new InvalidUserCodeException(exceptionMessage, e);
        }
    }
}
