/*
 * Copyright 2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.artifacts.ivyservice.ivyresolve;

import org.gradle.internal.component.external.model.ModuleComponentResolveMetadata;
import org.gradle.internal.reflect.Instantiator;

/**
 * A ModuleComponentRepository providing some user-configuration values.
 */
public interface ConfiguredModuleComponentRepository extends ModuleComponentRepository<ModuleComponentResolveMetadata> {

    /**
     * Should the 'dynamic resolve mode' be used for this (Ivy) repository, where the 'revConstraint'
     * attribute is used for versions instead of the 'rev' attribute.
     */
    boolean isDynamicResolveMode();

    /**
     * True for a repository that only accesses the local filesystem, and does not require any remote/network access.
     */
    boolean isLocal();

    void setComponentResolvers(ComponentResolvers resolver);

    Instantiator getComponentMetadataInstantiator();
}
