/*
 * Copyright 2012 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.tasks.testing.junit.result;

import org.gradle.api.Action;

import java.io.File;

public class BinaryResultBackedTestResultsProvider extends TestOutputStoreBackedResultsProvider {
    private final TestResultSerializer resultSerializer;

    public BinaryResultBackedTestResultsProvider(File resultsDir) {
        super(new TestOutputStore(resultsDir));
        this.resultSerializer = new TestResultSerializer(resultsDir);
    }

    @Override
    public void visitClasses(final Action<? super TestClassResult> visitor) {
        resultSerializer.read(visitor);
    }

    @Override
    public boolean isHasResults() {
        return resultSerializer.isHasResults();
    }
}
