/*
 * Copyright 2017 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.deployment.internal;

import javax.inject.Inject;

/**
 * Controls starting and stopping a deployment.
 *
 * Implementations of this interface should annotate at least one constructor with {@link Inject}, if
 * the implementation requires parameters.
 *
 * @since 4.2
 */
public interface DeploymentHandle {
    /**
     * Returns true if the deployment is still running.
     */
    boolean isRunning();

    /**
     * Starts the given deployment.
     * @param deployment the deployment to be started
     */
    void start(Deployment deployment);

    /**
     * Stops the deployment.
     */
    void stop();
}
