/*
 * Copyright 2020 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.plugins.ide.eclipse.model;

import org.gradle.api.artifacts.component.ComponentSelector;

/**
 * A library that turned out to be unresolved.
 *
 * @since 6.7
 */
public class UnresolvedLibrary extends Library {

    private ComponentSelector attemptedSelector;

    public UnresolvedLibrary(FileReference library) {
        super(library);
    }

    /**
     * The attempted selector reported by the related UnresolvedDependencyResult
     */
    public ComponentSelector getAttemptedSelector() {
        return attemptedSelector;
    }

    public void setAttemptedSelector(ComponentSelector attemptedSelector) {
        this.attemptedSelector = attemptedSelector;
    }
}
