/*
 * Copyright 2015 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.tooling.internal.protocol.test;

import org.gradle.tooling.internal.protocol.InternalProtocolInterface;
import org.gradle.tooling.internal.protocol.events.InternalTestDescriptor;

import java.util.Collection;

/**
 * DO NOT CHANGE THIS INTERFACE. It is part of the cross-version protocol.
 *
 * See {@code ProviderInternalTestExecutionRequest} for what the provider expects on a test execution request.
 *
 * @since 2.6-rc-1
 */
public interface InternalTestExecutionRequest extends InternalProtocolInterface {
    Collection<InternalTestDescriptor> getTestExecutionDescriptors();
    Collection<String> getTestClassNames();
    /**
     * @since 7.6
     */
    boolean isRunDefaultTasks();
}
