/*
 * Copyright 2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.nativeplatform.internal;

import org.gradle.internal.operations.logging.BuildOperationLogger;
import org.gradle.language.base.internal.compile.CompileSpec;
import org.gradle.nativeplatform.platform.NativePlatform;

import java.io.File;
import java.util.List;

public interface BinaryToolSpec extends CompileSpec {
    NativePlatform getTargetPlatform();

    void setTargetPlatform(NativePlatform platform);

    File getTempDir();

    void setTempDir(File tempDir);

    List<String> getArgs();

    void args(List<String> args);

    List<String> getSystemArgs();

    void systemArgs(List<String> args);

    List<String> getAllArgs();

    BuildOperationLogger getOperationLogger();

    void setOperationLogger(BuildOperationLogger oplogger);

}
