/*
 * Copyright 2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.resource.transport.http;

import org.apache.http.annotation.Contract;
import org.apache.http.annotation.ThreadingBehavior;
import org.apache.http.auth.AuthScheme;
import org.apache.http.protocol.HttpContext;

import java.nio.charset.Charset;

@Contract(threading = ThreadingBehavior.IMMUTABLE)
@SuppressWarnings("deprecation")
public class HttpHeaderSchemeFactory implements org.apache.http.auth.AuthSchemeFactory, org.apache.http.auth.AuthSchemeProvider {

    public HttpHeaderSchemeFactory(final Charset charset) {
        super();
    }

    public HttpHeaderSchemeFactory() {
        this(null);
    }

    @Override
    public AuthScheme newInstance(final org.apache.http.params.HttpParams params) {
        return new HttpHeaderAuthScheme();
    }

    @Override
    public AuthScheme create(final HttpContext context) {
        return new HttpHeaderAuthScheme();
    }

}
