/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.declarative.dsl.schema

import org.gradle.declarative.dsl.schema.DataType.ParameterizedTypeInstance.TypeArgument
import org.gradle.declarative.dsl.schema.DataTypeRef.Name
import org.gradle.declarative.dsl.schema.DataTypeRef.NameWithArgs
import org.gradle.declarative.dsl.schema.DataTypeRef.Type
import org.gradle.tooling.ToolingModelContract
import java.io.Serializable


@ToolingModelContract(subTypes = [
    Type::class,
    Name::class,
    NameWithArgs::class
])
sealed interface DataTypeRef : Serializable {
    interface Type : DataTypeRef {
        val dataType: DataType.PrimitiveType
    }

    interface Name : DataTypeRef {
        val fqName: FqName
    }

    interface NameWithArgs : DataTypeRef {
        val fqName: FqName
        val typeArguments: List<TypeArgument>
    }
}
