/*
 * Copyright 2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.process.internal.worker.request;

/**
 * Protocol for controlling a worker process.
 * <p>
 * This protocol is used by the daemon to control the current work the worker process is doing.
 */
public interface RequestProtocol {
    /**
     * Runs the given request in the worker process.
     * <p>
     * If the worker process can handle multiple requests, the worker process will continue running after the request is completed.
     *
     * @param request The request to run.
     */
    void run(Request request);

    /**
     * Runs the given request in the worker process, then stops the worker process.
     * <p>
     * If the worker process can handle multiple requests, the worker process will stop after the request is completed.
     *
     * @param request The request to run.
     */
    void runThenStop(Request request);

    /**
     * Stops the worker process.
     */
    void stop();
}
