/*
 * Copyright 2021 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.api.internal.catalog;

import javax.annotation.Nullable;
import java.io.Serializable;

public class AbstractContextAwareModel implements Serializable {
    protected final String context;

    public AbstractContextAwareModel(@Nullable String context) {
        this.context = context;
    }

    /**
     * The context where this dependency model was built from, used
     * for debugging and giving hints to the user.
     * It's intentionally not part of the identity.
     *
     * @return the context
     */
    @Nullable
    public String getContext() {
        return context;
    }
}
