/*
 * Copyright 2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.build.event.types;

import org.gradle.api.NonNullApi;
import org.gradle.internal.operations.OperationIdentifier;
import org.gradle.tooling.internal.protocol.events.InternalProblemDescriptor;

import javax.annotation.Nullable;
import java.io.Serializable;

@NonNullApi
public class DefaultProblemDescriptor implements Serializable, InternalProblemDescriptor {

    private final OperationIdentifier id;
    private final OperationIdentifier parentId;

    public DefaultProblemDescriptor(OperationIdentifier id, @Nullable OperationIdentifier parentId) {
        this.id = id;
        this.parentId = parentId;
    }

    @Override
    public Object getId() {
        return id;
    }

    @Override
    public String getName() {
        return "Problem";
    }

    @Override
    @Nullable
    public Object getParentId() {
        return parentId;
    }

    @Override
    public String getDisplayName() {
        return getName();
    }
}
