/*
 * Copyright 2017 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.nativeplatform.toolchain.internal.msvcpp.version;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;

public abstract class AbstractVisualStudioVersionLocator implements VisualStudioVersionLocator {
    private final List<VisualStudioInstallCandidate> installs = new ArrayList<>();
    private boolean initialized;

    @Nonnull
    @Override
    public List<VisualStudioInstallCandidate> getVisualStudioInstalls() {
        if (!initialized) {
            installs.addAll(locateInstalls());
            initialized = true;
        }

        return installs;
    }

    protected abstract List<VisualStudioInstallCandidate> locateInstalls();
}
