/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.declarativedsl.evaluator.defaults

import org.gradle.declarative.dsl.evaluation.InterpretationStepFeature
import org.gradle.internal.declarativedsl.analysis.ResolutionResult
import org.gradle.internal.declarativedsl.evaluator.features.ResolutionResultHandler
import java.io.Serializable


class DefineModelDefaults : InterpretationStepFeature.ResolutionResultPostprocessing.DefineModelDefaults, Serializable


class ModelDefaultsDefinitionCollector(private val defaultsRegistrar: ModelDefaultsDefinitionRegistrar) : ResolutionResultHandler {
    override fun shouldHandleFeature(feature: InterpretationStepFeature.ResolutionResultPostprocessing) =
        feature is InterpretationStepFeature.ResolutionResultPostprocessing.DefineModelDefaults

    override fun processResolutionResult(resolutionResult: ResolutionResult): ResolutionResult {
        defaultsRegistrar.registerDefaults(ModelDefaultsResolutionProcessor.process(resolutionResult))
        return resolutionResult
    }
}


interface ModelDefaultsDefinitionRegistrar {
    fun registerDefaults(modelDefaultsBySoftwareType: Map<String, ModelDefaultsResolutionResults>)
}
