/*
 * Copyright 2016 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.logging.services;

import org.gradle.internal.logging.events.OutputEventListener;
import org.gradle.internal.logging.events.ProgressCompleteEvent;
import org.gradle.internal.logging.events.ProgressEvent;
import org.gradle.internal.logging.events.ProgressStartEvent;
import org.gradle.internal.logging.progress.ProgressListener;

public class ProgressLoggingBridge implements ProgressListener {
    private final OutputEventListener listener;

    public ProgressLoggingBridge(OutputEventListener listener) {
        this.listener = listener;
    }

    @Override
    public void completed(ProgressCompleteEvent event) {
        listener.onOutput(event);
    }

    @Override
    public void started(ProgressStartEvent event) {
        listener.onOutput(event);
    }

    @Override
    public void progress(ProgressEvent event) {
        listener.onOutput(event);
    }
}
