/*
 * Copyright 2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.provider;

import org.gradle.api.Action;

import java.util.ArrayList;
import java.util.List;

public class ChangingValueHandler<T> implements ChangingValue<T> {
    private List<Action<T>> handlers;

    @Override
    public void onValueChange(Action<T> action) {
        if (handlers == null) {
            handlers = new ArrayList<>();
        }
        handlers.add(action);
    }

    public void handle(T previousValue) {
        if (handlers != null) {
            for (Action<T> handler : handlers) {
                handler.execute(previousValue);
            }
        }
    }
}
