/*
 * Copyright 2016 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.model.internal.inspect;

import java.lang.reflect.Method;

public interface RuleSourceValidationProblemCollector extends ValidationProblemCollector {
    void add(MethodRuleDefinition<?, ?> method, String problem);

    void add(MethodRuleDefinition<?, ?> method, String problem, Throwable cause);

    void add(Method method, String problem);
}
