/*
 * Copyright 2022 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.cc.impl

import org.gradle.internal.buildtree.BuildTreeWorkGraph
import org.gradle.internal.buildtree.BuildTreeWorkGraphPreparer


class ConfigurationCacheAwareBuildTreeWorkGraphPreparer(
    private val delegate: BuildTreeWorkGraphPreparer,
    private val cache: BuildTreeConfigurationCache
) : BuildTreeWorkGraphPreparer {
    override fun prepareToScheduleTasks(workGraph: BuildTreeWorkGraph.Builder) {
        if (!cache.isLoaded) {
            delegate.prepareToScheduleTasks(workGraph)
        } // else, not required
    }
}
