/*
 * Copyright 2018 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.vcs.internal;

/**
 * Represents how a version of a software component is referred to in a version
 * control system.
 *
 * @since 4.4
 */
public interface VersionRef {
    /**
     * Returns a human-readable version of the software component.
     */
    String getVersion();

    /**
     * Returns a canonical representation of this version of the software
     * component, which may be different from the human-readable version.
     */
    String getCanonicalId();
}
