/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.declarativedsl.dom.mutation

import org.gradle.declarative.dsl.schema.DataClass
import org.gradle.declarative.dsl.schema.DataProperty
import org.gradle.declarative.dsl.schema.DataType
import org.gradle.declarative.dsl.schema.SchemaMemberFunction


sealed interface TypedMember {
    val ownerType: DataType.ClassDataType

    data class TypedProperty(override val ownerType: DataClass, val property: DataProperty) : TypedMember {
        init {
            require(property in ownerType.properties) { "$property not found in the ownerType $ownerType" }
        }
    }

    data class TypedFunction(override val ownerType: DataClass, val function: SchemaMemberFunction) : TypedMember {
        init {
            require(function in ownerType.memberFunctions) { "$function not found in the ownerType $ownerType" }
        }
    }
}
