/*
 * Copyright 2010 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.initialization;

import java.io.File;
import java.util.Collection;

public class UserHomeInitScriptFinder extends DirectoryInitScriptFinder implements InitScriptFinder {

    private final File userHomeDir;

    public UserHomeInitScriptFinder(File userHomeDir) {
        this.userHomeDir = userHomeDir;
    }

    @Override
    public void findScripts(Collection<File> scripts) {
        File userInitScript = resolveScriptFile(userHomeDir, "init");
        if (userInitScript != null) {
            scripts.add(userInitScript);
        }
        findScriptsInDir(new File(userHomeDir, "init.d"), scripts);
    }
}

