/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.component.resolution.failure.interfaces;

/**
 * Represents a specific type of {@link VariantSelectionFailure} where the failure occurred
 * due to a failure to select a variant of a component based on the requested configuration name.
 */
public interface VariantSelectionByNameFailure extends VariantSelectionFailure {
    /**
     * Gets the name of the configuration that was requested directly by name.
     *
     * @return the name of the configuration that was requested
     */
    String getRequestedConfigurationName();
}
