/*
 * Copyright 2024 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.internal.initialization.transform.utils;

import org.gradle.api.internal.initialization.transform.InstrumentationArtifactMetadata;
import org.gradle.api.internal.initialization.transform.InstrumentationDependencyAnalysis;

import java.io.File;
import java.util.Map;
import java.util.Set;

public interface InstrumentationAnalysisSerializer {
    void writeDependencyAnalysis(File output, InstrumentationDependencyAnalysis dependencyAnalysis);

    void writeTypeHierarchyAnalysis(File output, Map<String, Set<String>> types);

    InstrumentationDependencyAnalysis readDependencyAnalysis(File output);

    InstrumentationArtifactMetadata readMetadataOnly(File input);

    Map<String, Set<String>> readTypeHierarchyAnalysis(File input);
}
