/*
 * Copyright 2022 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.jvm.inspection;

import org.gradle.jvm.toolchain.internal.InstallationLocation;

import javax.annotation.Nonnull;
import java.util.function.BiConsumer;

public class ReportingJvmMetadataDetector implements JvmMetadataDetector {

    @Nonnull
    private final JvmMetadataDetector delegate;

    @Nonnull
    private final BiConsumer<InstallationLocation, JvmInstallationMetadata> reporter;

    public ReportingJvmMetadataDetector(@Nonnull JvmMetadataDetector delegate, @Nonnull BiConsumer<InstallationLocation, JvmInstallationMetadata> reporter) {
        this.delegate = delegate;
        this.reporter = reporter;
    }

    @Override
    public JvmInstallationMetadata getMetadata(InstallationLocation javaInstallationLocation) {
        JvmInstallationMetadata delegateResult = delegate.getMetadata(javaInstallationLocation);
        reporter.accept(javaInstallationLocation, delegateResult);
        return delegateResult;
    }
}
