/*
 * Copyright 2023 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.api.problems.internal;

import com.google.common.base.Objects;
import com.google.common.base.Preconditions;

public class DefaultDocLink implements InternalDocLink {

    private final String url;

    public DefaultDocLink(String url) {
        this.url = Preconditions.checkNotNull(url);
    }

    @Override
    public String getUrl() {
        return url;
    }

    @Override
    public String getConsultDocumentationMessage() {
        return "For more information, please refer to " + url + ".";
    }

    @Override
    public boolean equals(Object o) {
        if (!(o instanceof DefaultDocLink)) {
            return false;
        }
        DefaultDocLink that = (DefaultDocLink) o;
        return Objects.equal(url, that.url);
    }

    @Override
    public int hashCode() {
        return Objects.hashCode(url);
    }

    @Override
    public String toString() {
        return "DefaultDocLink{" +
            "url='" + url + '\'' +
            '}';
    }
}
