/*
 * Copyright 2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.io;

import javax.annotation.Nullable;

public interface TextStream {
    /**
     * Called when some chunk of text is available.
     */
    void text(String text);

    /**
     * Called when the end of the stream is reached for some reason.
     *
     * @param failure The failure, if any, which caused the end of the stream.
     */
    void endOfStream(@Nullable Throwable failure);
}
