/*
 * Copyright 2017 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.snapshot.impl;

import org.gradle.internal.exceptions.Contextual;
import org.gradle.internal.logging.text.TreeFormatter;

/**
 * Represents a problem while attempting to isolate an instance.
 */
@Contextual
public class IsolationException extends RuntimeException {
    public IsolationException(Object value) {
        super(format(value));
    }

    public IsolationException(Object value, Throwable cause) {
        super(format(value), cause);
    }

    private static String format(Object value) {
        TreeFormatter formatter = new TreeFormatter();
        formatter.node("Could not isolate value ");
        formatter.appendValue(value);
        formatter.append(" of type ");
        formatter.appendType(value.getClass());
        return formatter.toString();
    }
}
