/*
 * Copyright 2014 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.internal.operations;

public final class OperationStartEvent {
    private final long startTime;

    /**
     * Create a new operation start event.
     * @param currentTime current time when the operation started
     */
    public OperationStartEvent(long currentTime) {
        this.startTime = currentTime;
    }

    public long getStartTime() {
        return startTime;
    }

    @Override
    public String toString() {
        return "OperationStartEvent{" +
            "startTime=" + startTime +
            '}';
    }
}
