/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.api.tasks.diagnostics;

import org.gradle.work.DisableCachingByDefault;

/**
 * Displays the dependency tree for a project. An instance of this type is used when you
 * execute the {@code dependencies} task from the command-line.
 */
@DisableCachingByDefault(because = "Not worth caching")
public abstract class DependencyReportTask extends AbstractDependencyReportTask {}
