/*
 * Copyright 2014 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.gradle.platform.base;

import org.gradle.api.Named;
import org.gradle.internal.HasInternalProtocol;

/**
 * A set of compilers that are used together to construct binaries.
 */
@HasInternalProtocol
public interface ToolChain extends Named {
    /**
     * Returns a human consumable name for this tool chain.
     *
     * @since 1.11
     */
    String getDisplayName();
}
